@@ -122,14 +122,14 @@ example_echo_factory_class_init (ExampleEchoFactoryClass *klass)
       "The connection that owns this channel factory",
       TP_TYPE_BASE_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB);
+      G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
   g_type_class_add_private (klass, sizeof (ExampleEchoFactoryPrivate));
 }
 
 static void
-close_all (TpChannelFactoryIface *iface)
+example_echo_factory_close_all (TpChannelFactoryIface *iface)
 {
   ExampleEchoFactory *self = EXAMPLE_ECHO_FACTORY (iface);
 
@@ -149,9 +149,9 @@ struct _ForeachData
 };
 
 static void
-_foreach (gpointer key,
-          gpointer value,
-          gpointer user_data)
+foreach_helper (gpointer key,
+                gpointer value,
+                gpointer user_data)
 {
   struct _ForeachData *data = user_data;
   TpChannelIface *chan = TP_CHANNEL_IFACE (value);
@@ -160,14 +160,14 @@ _foreach (gpointer key,
 }
 
 static void
-foreach (TpChannelFactoryIface *iface,
-         TpChannelFunc callback,
-         gpointer user_data)
+example_echo_factory_foreach (TpChannelFactoryIface *iface,
+                              TpChannelFunc callback,
+                              gpointer user_data)
 {
   ExampleEchoFactory *self = EXAMPLE_ECHO_FACTORY (iface);
   struct _ForeachData data = { user_data, callback };
 
-  g_hash_table_foreach (self->priv->channels, _foreach, &data);
+  g_hash_table_foreach (self->priv->channels, foreach_helper, &data);
 }
 
 static void
@@ -213,13 +213,13 @@ new_channel (ExampleEchoFactory *self, TpHandle handle)
 }
 
 static TpChannelFactoryRequestStatus
-request (TpChannelFactoryIface *iface,
-         const gchar *chan_type,
-         TpHandleType handle_type,
-         guint handle,
-         gpointer request_id,
-         TpChannelIface **ret,
-         GError **error)
+example_echo_factory_request (TpChannelFactoryIface *iface,
+                              const gchar *chan_type,
+                              TpHandleType handle_type,
+                              guint handle,
+                              gpointer request_id,
+                              TpChannelIface **ret,
+                              GError **error)
 {
   ExampleEchoFactory *self = EXAMPLE_ECHO_FACTORY (iface);
   ExampleEchoChannel *chan;
@@ -256,7 +256,7 @@ iface_init (gpointer iface,
 {
   TpChannelFactoryIfaceClass *klass = iface;
 
-  klass->close_all = close_all;
-  klass->foreach = foreach;
-  klass->request = request;
+  klass->close_all = example_echo_factory_close_all;
+  klass->foreach = example_echo_factory_foreach;
+  klass->request = example_echo_factory_request;
 }
