@@ -92,45 +92,22 @@ tp_errors_disconnected_quark (void)
 /**
  * TpConnectionClass:
  * @parent_class: the parent class
- * @priv: pointer to opaque private data
  *
- * The class of a #TpConnection.
+ * The class of a #TpConnection. In addition to @parent_class there are four
+ * pointers reserved for possible future use.
  *
- * Since: 0.7.1
+ * Since: 0.7.1; structure layout visible since 0.7.12
  */
-struct _TpConnectionClass {
-    TpProxyClass parent_class;
-    gpointer priv;
-};
 
 /**
  * TpConnection:
  * @parent: the parent class instance
- * @status: same as #TpConnection:status, should be considered read-only
- * @status_reason: same as #TpConnection:status-reason, should be considered
- *  read-only
- * @_reserved_for_self_handle: reserved, currently always 0
- * @ready: the same as #TpChannel:channel-ready; should be considered
- *  read-only
- * @_reserved_flags: (private, reserved for future use)
  * @priv: pointer to opaque private data
  *
  * A proxy object for a Telepathy connection.
  *
- * Since: 0.7.1
+ * Since: 0.7.1; structure layout visible since 0.7.12
  */
-struct _TpConnection {
-    TpProxy parent;
-
-    TpConnectionStatus status;
-    TpConnectionStatusReason status_reason;
-    TpHandle _reserved_for_self_handle;
-
-    gboolean ready:1;
-    gboolean _reserved_flags:31;
-
-    TpConnectionPrivate *priv;
-};
 
 typedef void (*TpConnectionProc) (TpConnection *self);
 
@@ -138,8 +115,11 @@ struct _TpConnectionPrivate {
     /* GArray of TpConnectionProc */
     GArray *introspect_needed;
 
+    TpConnectionStatus status;
+    TpConnectionStatusReason status_reason;
     TpConnectionAliasFlags alias_flags;
-    /* other stuff could go here */
+
+    gboolean ready:1;
 };
 
 enum
@@ -165,13 +145,13 @@ tp_connection_get_property (GObject *object,
   switch (property_id)
     {
     case PROP_CONNECTION_READY:
-      g_value_set_boolean (value, self->ready);
+      g_value_set_boolean (value, self->priv->ready);
       break;
     case PROP_STATUS:
-      g_value_set_uint (value, self->status);
+      g_value_set_uint (value, self->priv->status);
       break;
     case PROP_STATUS_REASON:
-      g_value_set_uint (value, self->status_reason);
+      g_value_set_uint (value, self->priv->status_reason);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -190,7 +170,7 @@ tp_connection_continue_introspection (TpConnection *self)
       self->priv->introspect_needed = NULL;
 
       DEBUG ("%p: connection ready", self);
-      self->ready = TRUE;
+      self->priv->ready = TRUE;
       g_object_notify ((GObject *) self, "connection-ready");
     }
   else
@@ -248,14 +228,15 @@ tp_connection_got_interfaces_cb (TpConnection *self,
     }
 
   DEBUG ("%p: Introspected interfaces", self);
+
+  g_assert (self->priv->introspect_needed == NULL);
+  self->priv->introspect_needed = g_array_new (FALSE, FALSE,
+      sizeof (TpConnectionProc));
+
   if (interfaces != NULL)
     {
       const gchar **iter;
 
-      g_assert (self->priv->introspect_needed == NULL);
-      self->priv->introspect_needed = g_array_new (FALSE, FALSE,
-          sizeof (TpConnectionProc));
-
       for (iter = interfaces; *iter != NULL; iter++)
         {
           if (tp_dbus_check_valid_interface_name (*iter, NULL))
@@ -301,6 +282,9 @@ tp_connection_got_interfaces_cb (TpConnection *self,
         }
     }
 
+  /* FIXME: give subclasses a chance to influence the definition of "ready"
+   * now that we have our interfaces? */
+
   tp_connection_continue_introspection (self);
 }
 
@@ -309,10 +293,10 @@ tp_connection_status_changed (TpConnection *self,
                               guint status,
                               guint reason)
 {
-  DEBUG ("%p: %d -> %d because %d", self, self->status, status, reason);
+  DEBUG ("%p: %d -> %d because %d", self, self->priv->status, status, reason);
 
-  self->status = status;
-  self->status_reason = reason;
+  self->priv->status = status;
+  self->priv->status_reason = reason;
   g_object_notify ((GObject *) self, "status");
   g_object_notify ((GObject *) self, "status-reason");
 
@@ -330,7 +314,14 @@ tp_connection_status_changed_cb (TpConnection *self,
                                  gpointer user_data,
                                  GObject *weak_object)
 {
-  tp_connection_status_changed (self, status, reason);
+  /* GetStatus is called in the TpConnection constructor. If we don't have the
+   * reply for this GetStatus call yet, ignore this signal StatusChanged in
+   * order to run the interface introspection only one time. We will get the
+   * GetStatus reply later anyway. */
+  if (self->priv->status != TP_UNKNOWN_CONNECTION_STATUS)
+    {
+      tp_connection_status_changed (self, status, reason);
+    }
 
   /* we only want to run this in response to a StatusChanged signal,
    * not if the initial status is DISCONNECTED */
@@ -401,8 +392,8 @@ tp_connection_init (TpConnection *self)
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, TP_TYPE_CONNECTION,
       TpConnectionPrivate);
 
-  self->status = TP_UNKNOWN_CONNECTION_STATUS;
-  self->status_reason = TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED;
+  self->priv->status = TP_UNKNOWN_CONNECTION_STATUS;
+  self->priv->status_reason = TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED;
 }
 
 static void
@@ -626,7 +617,7 @@ tp_connection_run_until_ready (TpConnection *self,
   if (as_proxy->invalidated)
     goto raise_invalidated;
 
-  if (self->ready)
+  if (self->priv->ready)
     return TRUE;
 
   data.loop = g_main_loop_new (NULL, FALSE);
@@ -636,7 +627,7 @@ tp_connection_run_until_ready (TpConnection *self,
   ready_id = g_signal_connect_swapped (self, "notify::connection-ready",
       G_CALLBACK (g_main_loop_quit), data.loop);
 
-  if (self->status != TP_CONNECTION_STATUS_CONNECTED &&
+  if (self->priv->status != TP_CONNECTION_STATUS_CONNECTED &&
       connect)
     {
       data.pc = tp_cli_connection_call_connect (self, -1,
@@ -671,7 +662,7 @@ tp_connection_run_until_ready (TpConnection *self,
   if (as_proxy->invalidated != NULL)
     goto raise_invalidated;
 
-  if (self->ready)
+  if (self->priv->ready)
     return TRUE;
 
   g_set_error (error, TP_DBUS_ERRORS, TP_DBUS_ERROR_CANCELLED,
@@ -859,3 +850,151 @@ tp_list_connection_names (TpDBusDaemon *bus_daemon,
       tp_list_connection_names_helper, list_context,
       list_context_free, weak_object);
 }
+
+static gpointer
+tp_connection_once (gpointer data G_GNUC_UNUSED)
+{
+  GType type = TP_TYPE_CONNECTION;
+
+  tp_proxy_init_known_interfaces ();
+
+  tp_proxy_or_subclass_hook_on_interface_add (type,
+      tp_cli_connection_add_signals);
+  tp_proxy_subclass_add_error_mapping (type,
+      TP_ERROR_PREFIX, TP_ERRORS, TP_TYPE_ERROR);
+
+  return NULL;
+}
+
+/**
+ * tp_connection_init_known_interfaces:
+ *
+ * Ensure that the known interfaces for TpConnection have been set up.
+ * This is done automatically when necessary, but for correct
+ * overriding of library interfaces by local extensions, you should
+ * call this function before calling
+ * tp_proxy_or_subclass_hook_on_interface_add() with first argument
+ * %TP_TYPE_CONNECTION.
+ *
+ * Since: 0.7.6
+ */
+void
+tp_connection_init_known_interfaces (void)
+{
+  static GOnce once = G_ONCE_INIT;
+
+  g_once (&once, tp_connection_once, NULL);
+}
+
+typedef struct {
+    TpConnectionWhenReadyCb callback;
+    gpointer user_data;
+    gulong invalidated_id;
+    gulong ready_id;
+} CallWhenReadyContext;
+
+static void
+cwr_invalidated (TpConnection *self,
+                 guint domain,
+                 gint code,
+                 gchar *message,
+                 gpointer user_data)
+{
+  CallWhenReadyContext *ctx = user_data;
+  GError e = { domain, code, message };
+
+  DEBUG ("enter");
+
+  g_assert (ctx->callback != NULL);
+
+  ctx->callback (self, &e, ctx->user_data);
+
+  g_signal_handler_disconnect (self, ctx->invalidated_id);
+  g_signal_handler_disconnect (self, ctx->ready_id);
+
+  ctx->callback = NULL;   /* poison it to detect errors */
+  g_slice_free (CallWhenReadyContext, ctx);
+}
+
+static void
+cwr_ready (TpConnection *self,
+           GParamSpec *unused G_GNUC_UNUSED,
+           gpointer user_data)
+{
+  CallWhenReadyContext *ctx = user_data;
+
+  DEBUG ("enter");
+
+  g_assert (ctx->callback != NULL);
+
+  ctx->callback (self, NULL, ctx->user_data);
+
+  g_signal_handler_disconnect (self, ctx->invalidated_id);
+  g_signal_handler_disconnect (self, ctx->ready_id);
+
+  ctx->callback = NULL;   /* poison it to detect errors */
+  g_slice_free (CallWhenReadyContext, ctx);
+}
+
+/**
+ * TpConnectionWhenReadyCb:
+ * @connection: the connection (which may be in the middle of being disposed,
+ *  if error is non-%NULL, error->domain is TP_DBUS_ERRORS and error->code is
+ *  TP_DBUS_ERROR_PROXY_UNREFERENCED)
+ * @error: %NULL if the connection is ready for use, or the error with which
+ *  it was invalidated if it is now invalid
+ * @user_data: whatever was passed to tp_connection_call_when_ready()
+ *
+ * Signature of a callback passed to tp_connection_call_when_ready(), which
+ * will be called exactly once, when the connection becomes ready or
+ * invalid (whichever happens first)
+ */
+
+/**
+ * tp_connection_call_when_ready:
+ * @self: a connection
+ * @callback: called when the connection becomes ready or invalidated,
+ *  whichever happens first
+ * @user_data: arbitrary user-supplied data passed to the callback
+ *
+ * If @self is ready for use or has been invalidated, call @callback
+ * immediately, then return. Otherwise, arrange
+ * for @callback to be called when @self either becomes ready for use
+ * or becomes invalid.
+ *
+ * Note that if the connection is not in state CONNECTED, the callback will
+ * not be called until the connection either goes to state CONNECTED
+ * or is invalidated (e.g. by going to state DISCONNECTED or by becoming
+ * unreferenced). In particular, this method does not call Connect().
+ * Call tp_cli_connection_call_connect() too, if you want to do that.
+ *
+ * Since: 0.7.7
+ */
+void
+tp_connection_call_when_ready (TpConnection *self,
+                               TpConnectionWhenReadyCb callback,
+                               gpointer user_data)
+{
+  TpProxy *as_proxy = (TpProxy *) self;
+
+  g_return_if_fail (callback != NULL);
+
+  if (self->priv->ready || as_proxy->invalidated != NULL)
+    {
+      DEBUG ("already ready or invalidated");
+      callback (self, as_proxy->invalidated, user_data);
+    }
+  else
+    {
+      CallWhenReadyContext *ctx = g_slice_new (CallWhenReadyContext);
+
+      DEBUG ("arranging callback later");
+
+      ctx->callback = callback;
+      ctx->user_data = user_data;
+      ctx->invalidated_id = g_signal_connect (self, "invalidated",
+          G_CALLBACK (cwr_invalidated), ctx);
+      ctx->ready_id = g_signal_connect (self, "notify::connection-ready",
+          G_CALLBACK (cwr_ready), ctx);
+    }
+}
