@@ -30,6 +30,20 @@ typedef struct _TpConnection TpConnection;
 typedef struct _TpConnectionPrivate TpConnectionPrivate;
 typedef struct _TpConnectionClass TpConnectionClass;
 
+struct _TpConnectionClass {
+    TpProxyClass parent_class;
+    /*<private>*/
+    GCallback _1;
+    GCallback _2;
+    GCallback _3;
+    GCallback _4;
+};
+
+struct _TpConnection {
+    TpProxy parent;
+    TpConnectionPrivate *priv;
+};
+
 GType tp_connection_get_type (void);
 
 #define TP_ERRORS_DISCONNECTED (tp_errors_disconnected_quark ())
@@ -61,6 +75,12 @@ gboolean tp_connection_run_until_ready (TpConnection *self,
     gboolean connect, GError **error,
     GMainLoop **loop);
 
+typedef void (*TpConnectionWhenReadyCb) (TpConnection *connection,
+    const GError *error, gpointer user_data);
+
+void tp_connection_call_when_ready (TpConnection *self,
+    TpConnectionWhenReadyCb callback, gpointer user_data);
+
 typedef void (*TpConnectionNameListCb) (const gchar * const *names,
     gsize n, const gchar * const *cms, const gchar * const *protocols,
     const GError *error, gpointer user_data,
@@ -71,6 +91,8 @@ void tp_list_connection_names (TpDBusDaemon *bus_daemon,
     gpointer user_data, GDestroyNotify destroy,
     GObject *weak_object);
 
+void tp_connection_init_known_interfaces (void);
+
 G_END_DECLS
 
 #include <telepathy-glib/_gen/tp-cli-connection.h>
