@@ -50,16 +50,22 @@
  * <literal>G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_GROUP,
  * tp_external_group_mixin_iface_init)</literal> in the fourth argument to
  * <literal>G_DEFINE_TYPE_WITH_CODE</literal>.
+ *
+ * Since 0.7.10 you can also implement the properties of Group channels,
+ * by calling tp_group_mixin_init_dbus_properties() or
+ * tp_external_group_mixin_init_dbus_properties() (as appropriate).
  */
 
 #include <telepathy-glib/group-mixin.h>
 
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
+#include <string.h>
 
 #include <telepathy-glib/debug-ansi.h>
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/gtypes.h>
+#include <telepathy-glib/interfaces.h>
 
 #define DEBUG_FLAG TP_DEBUG_GROUPS
 
@@ -107,7 +113,8 @@ local_pending_info_new (TpHandleRepoIface *repo,
   info->reason = reason;
   info->message = g_strdup (message);
   info->repo = repo;
-  if (actor)
+
+  if (actor != 0)
     tp_handle_ref (repo, actor);
 
   return info;
@@ -117,8 +124,10 @@ static void
 local_pending_info_free (LocalPendingInfo *info)
 {
   g_free ((gchar *)info->message);
-  if (info->actor)
+
+  if (info->actor != 0)
     tp_handle_unref (info->repo, info->actor);
+
   g_slice_free (LocalPendingInfo, info);
 }
 
@@ -256,6 +265,9 @@ tp_group_mixin_init (GObject *obj,
   mixin->handle_repo = handle_repo;
   mixin->self_handle = self_handle;
 
+  if (mixin->self_handle != 0)
+    tp_handle_ref (handle_repo, mixin->self_handle);
+
   mixin->group_flags = 0;
 
   mixin->members = tp_handle_set_new (handle_repo);
@@ -298,7 +310,9 @@ handle_owners_foreach_unref (gpointer key,
   TpGroupMixin *mixin = user_data;
 
   tp_handle_unref (mixin->handle_repo, GPOINTER_TO_UINT (key));
-  tp_handle_unref (mixin->handle_repo, GPOINTER_TO_UINT (value));
+
+  if (GPOINTER_TO_UINT (value) != 0)
+    tp_handle_unref (mixin->handle_repo, GPOINTER_TO_UINT (value));
 }
 
 /**
@@ -326,6 +340,9 @@ tp_group_mixin_finalize (GObject *obj)
 
   g_slice_free (TpGroupMixinPrivate, mixin->priv);
 
+  if (mixin->self_handle != 0)
+    tp_handle_unref (mixin->handle_repo, mixin->self_handle);
+
   tp_handle_set_destroy (mixin->members);
   tp_handle_set_destroy (mixin->local_pending);
   tp_handle_set_destroy (mixin->remote_pending);
@@ -363,6 +380,37 @@ tp_group_mixin_get_self_handle (GObject *obj,
   return TRUE;
 }
 
+
+/**
+ * tp_group_mixin_change_self_handle:
+ * @obj: An object implementing the group interface using this mixin
+ * @new_self_handle: The new self-handle for this group
+ *
+ * Change the self-handle for this group to the given value.
+ */
+void
+tp_group_mixin_change_self_handle (GObject *obj,
+                                   TpHandle new_self_handle)
+{
+  TpGroupMixin *mixin = TP_GROUP_MIXIN (obj);
+  TpHandle old_self_handle = mixin->self_handle;
+
+  DEBUG ("%u '%s'", new_self_handle,
+      tp_handle_inspect (mixin->handle_repo, new_self_handle));
+
+  if (new_self_handle != 0)
+    tp_handle_ref (mixin->handle_repo, new_self_handle);
+
+  mixin->self_handle = new_self_handle;
+
+  tp_svc_channel_interface_group_emit_self_handle_changed (obj,
+      new_self_handle);
+
+  if (old_self_handle != 0)
+    tp_handle_unref (mixin->handle_repo, old_self_handle);
+}
+
+
 static void
 tp_group_mixin_get_self_handle_async (TpSvcChannelInterfaceGroup *obj,
                                       DBusGMethodInvocation *context)
@@ -1079,10 +1127,13 @@ group_flags_to_string (TpChannelGroupFlags flags)
  * @add: Flags to be added
  * @del: Flags to be removed
  *
- * Request a change to be made to the flags. Emits the
- * GroupFlagsChanged signal with the changes which were actually made.
+ * Request a change to be made to the flags. If any flags were actually
+ * set or cleared, emits the GroupFlagsChanged signal with the changes.
  *
  * It is an error to set any of the same bits in both @add and @del.
+ *
+ * Changed in 0.7.7: the signal is not emitted if adding @add and
+ *  removing @del had no effect on the existing group flags.
  */
 void
 tp_group_mixin_change_flags (GObject *obj,
@@ -1101,7 +1152,12 @@ tp_group_mixin_change_flags (GObject *obj,
   removed = del & mixin->group_flags;
   mixin->group_flags &= ~removed;
 
-  if (add != 0 || del != 0)
+  if (added == 0 && removed == 0)
+    {
+      DEBUG ("No change: %u includes all the bits of %u and none of %u",
+          mixin->group_flags, add, del);
+    }
+  else
     {
       gchar *str_added, *str_removed, *str_flags;
 
@@ -1168,8 +1224,8 @@ member_array_to_string (TpHandleRepoIface *repo,
   return g_string_free (str, FALSE);
 }
 
-static void remove_handle_owners_if_exist (GObject *obj,
-    GArray *array);
+static GArray *remove_handle_owners_if_exist (GObject *obj,
+    GArray *array) G_GNUC_WARN_UNUSED_RESULT;
 
 typedef struct {
     TpGroupMixin *mixin;
@@ -1361,7 +1417,7 @@ tp_group_mixin_change_members (GObject *obj,
       tp_intset_size (new_remote_pending) > 0)
     {
       GArray *arr_add, *arr_remove, *arr_local, *arr_remote;
-      gchar *add_str, *rem_str, *local_str, *remote_str;
+      GArray *arr_owners_removed;
 
       /* translate intsets to arrays */
       arr_add = tp_intset_to_array (new_add);
@@ -1370,10 +1426,12 @@ tp_group_mixin_change_members (GObject *obj,
       arr_remote = tp_intset_to_array (new_remote_pending);
 
       /* remove any handle owner mappings */
-      remove_handle_owners_if_exist (obj, arr_remove);
+      arr_owners_removed = remove_handle_owners_if_exist (obj, arr_remove);
 
       if (DEBUGGING)
         {
+          gchar *add_str, *rem_str, *local_str, *remote_str;
+
           add_str = member_array_to_string (mixin->handle_repo, arr_add);
           rem_str = member_array_to_string (mixin->handle_repo, arr_remove);
           local_str = member_array_to_string (mixin->handle_repo, arr_local);
@@ -1416,11 +1474,36 @@ tp_group_mixin_change_members (GObject *obj,
             }
         }
 
+      if (arr_owners_removed->len > 0)
+        {
+          GHashTable *empty_hash_table = g_hash_table_new (NULL, NULL);
+
+          tp_svc_channel_interface_group_emit_handle_owners_changed (obj,
+              empty_hash_table, arr_owners_removed);
+
+          if (mixin->priv->externals != NULL)
+            {
+              guint i;
+
+              for (i = 0; i < mixin->priv->externals->len; i++)
+                {
+                  tp_svc_channel_interface_group_emit_handle_owners_changed (
+                      g_ptr_array_index (mixin->priv->externals, i),
+                      empty_hash_table, arr_owners_removed);
+                }
+            }
+
+          tp_handles_unref (mixin->handle_repo, arr_owners_removed);
+
+          g_hash_table_destroy (empty_hash_table);
+        }
+
       /* free arrays */
       g_array_free (arr_add, TRUE);
       g_array_free (arr_remove, TRUE);
       g_array_free (arr_local, TRUE);
       g_array_free (arr_remote, TRUE);
+      g_array_free (arr_owners_removed, TRUE);
 
       ret = TRUE;
     }
@@ -1445,56 +1528,144 @@ tp_group_mixin_change_members (GObject *obj,
  * tp_group_mixin_add_handle_owner:
  * @obj: A GObject implementing the group interface with this mixin
  * @local_handle: A contact handle valid within this group (may not be 0)
- * @owner_handle: A contact handle valid globally (may not be 0)
+ * @owner_handle: A contact handle valid globally, or 0 if the owner of the
+ *  @local_handle is unknown
  *
  * Note that the given local handle is an alias within this group
  * for the given globally-valid handle. It will be returned from subsequent
  * GetHandleOwner queries where appropriate.
+ *
+ * Changed in 0.7.10: The @owner_handle may be 0. To comply with telepathy-spec
+ *  0.17.6, before adding any channel-specific handle to the members,
+ *  local-pending members or remote-pending members, you must call either
+ *  this function or tp_group_mixin_add_handle_owners().
  */
 void
 tp_group_mixin_add_handle_owner (GObject *obj,
                                  TpHandle local_handle,
                                  TpHandle owner_handle)
 {
-  TpGroupMixin *mixin = TP_GROUP_MIXIN (obj);
-  TpGroupMixinPrivate *priv = mixin->priv;
+  GHashTable *tmp;
 
   g_return_if_fail (local_handle != 0);
-  g_return_if_fail (owner_handle != 0);
 
-  g_hash_table_insert (priv->handle_owners, GUINT_TO_POINTER (local_handle),
-                       GUINT_TO_POINTER (owner_handle));
+  tmp = g_hash_table_new (g_direct_hash, g_direct_equal);
+  g_hash_table_insert (tmp, GUINT_TO_POINTER (local_handle),
+      GUINT_TO_POINTER (owner_handle));
 
-  tp_handle_ref (mixin->handle_repo, local_handle);
-  tp_handle_ref (mixin->handle_repo, owner_handle);
+  tp_group_mixin_add_handle_owners (obj, tmp);
+
+  g_hash_table_destroy (tmp);
 }
 
+
 static void
+add_handle_owners_helper (gpointer key,
+                          gpointer value,
+                          gpointer user_data)
+{
+  TpHandle local_handle = GPOINTER_TO_UINT (key);
+  TpHandle owner_handle = GPOINTER_TO_UINT (value);
+  TpGroupMixin *mixin = user_data;
+  gpointer orig_key, orig_value;
+
+  g_return_if_fail (local_handle != 0);
+
+  tp_handle_ref (mixin->handle_repo, local_handle);
+
+  if (owner_handle != 0)
+    tp_handle_ref (mixin->handle_repo, owner_handle);
+
+  if (g_hash_table_lookup_extended (mixin->priv->handle_owners, key,
+        &orig_key, &orig_value))
+    {
+      /* no need to actually remove - we're about to overwrite them anyway */
+
+      tp_handle_unref (mixin->handle_repo, local_handle);
+
+      if (GPOINTER_TO_UINT (orig_value) != 0)
+        tp_handle_unref (mixin->handle_repo, GPOINTER_TO_UINT (orig_value));
+    }
+
+  g_hash_table_insert (mixin->priv->handle_owners, key, value);
+}
+
+
+/**
+ * tp_group_mixin_add_handle_owners:
+ * @obj: A GObject implementing the group interface with this mixin
+ * @local_to_owner_handle: A map from contact handles valid within this group
+ *  (which may not be 0) to either contact handles valid globally, or 0 if the
+ *  owner of the corresponding key is unknown; all handles are stored using
+ *  GUINT_TO_POINTER
+ *
+ * Note that the given local handles are aliases within this group
+ * for the given globally-valid handles.
+ *
+ * To comply with telepathy-spec 0.17.6, before adding any channel-specific
+ * handle to the members, local-pending members or remote-pending members, you
+ * must call either this function or tp_group_mixin_add_handle_owner().
+ *
+ * @since 0.7.10
+ */
+void
+tp_group_mixin_add_handle_owners (GObject *obj,
+                                  GHashTable *local_to_owner_handle)
+{
+  TpGroupMixin *mixin = TP_GROUP_MIXIN (obj);
+  GArray *empty_array;
+
+  if (g_hash_table_size (local_to_owner_handle) == 0)
+    return;
+
+  empty_array = g_array_sized_new (FALSE, FALSE, sizeof (guint), 0);
+
+  g_hash_table_foreach (local_to_owner_handle, add_handle_owners_helper,
+      mixin);
+
+  tp_svc_channel_interface_group_emit_handle_owners_changed (obj,
+      local_to_owner_handle, empty_array);
+
+  g_array_free (empty_array, TRUE);
+}
+
+
+static GArray *
 remove_handle_owners_if_exist (GObject *obj,
                                GArray *array)
 {
   TpGroupMixin *mixin = TP_GROUP_MIXIN (obj);
   TpGroupMixinPrivate *priv = mixin->priv;
   guint i;
+  GArray *ret;
+
+  ret = g_array_sized_new (FALSE, FALSE, sizeof (guint), array->len);
 
   for (i = 0; i < array->len; i++)
     {
       TpHandle handle = g_array_index (array, guint, i);
       gpointer local_handle, owner_handle;
 
+      g_assert (handle != 0);
+
       if (g_hash_table_lookup_extended (priv->handle_owners,
                                         GUINT_TO_POINTER (handle),
                                         &local_handle,
                                         &owner_handle))
         {
-          tp_handle_unref (mixin->handle_repo,
-              GPOINTER_TO_UINT (local_handle));
-          tp_handle_unref (mixin->handle_repo,
-              GPOINTER_TO_UINT (owner_handle));
+          g_assert (GPOINTER_TO_UINT (local_handle) == handle);
+          g_array_append_val (ret, handle);
+          /* don't unref local_handle - ownership is transferred to ret */
+
+          if (GPOINTER_TO_UINT (owner_handle) != 0)
+            tp_handle_unref (mixin->handle_repo,
+                GPOINTER_TO_UINT (owner_handle));
 
           g_hash_table_remove (priv->handle_owners, GUINT_TO_POINTER (handle));
         }
     }
+
+  return ret;
 }
 
 /**
@@ -1528,6 +1699,148 @@ tp_group_mixin_iface_init (gpointer g_iface, gpointer iface_data)
 #undef IMPLEMENT
 }
 
+
+enum {
+    MIXIN_DP_GROUP_FLAGS,
+    MIXIN_DP_HANDLE_OWNERS,
+    MIXIN_DP_LOCAL_PENDING_MEMBERS,
+    MIXIN_DP_MEMBERS,
+    MIXIN_DP_REMOTE_PENDING_MEMBERS,
+    MIXIN_DP_SELF_HANDLE,
+    NUM_MIXIN_DBUS_PROPERTIES
+};
+
+
+/**
+ * tp_group_mixin_get_dbus_property:
+ * @object: An object with this mixin
+ * @interface: Must be %TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP
+ * @name: A quark representing the D-Bus property name, either
+ *  "GroupFlags", "HandleOwners", "LocalPendingMembers", "Members",
+ *  "RemotePendingMembers" or "SelfHandle"
+ * @value: A GValue pre-initialized to the right type, into which to put the
+ *  value
+ * @unused: Ignored
+ *
+ * An implementation of #TpDBusPropertiesMixinGetter which assumes that the
+ * @object has the group mixin. It can only be used for the Group interface.
+ *
+ * Since: 0.7.10
+ */
+void
+tp_group_mixin_get_dbus_property (GObject *object,
+                                  GQuark interface,
+                                  GQuark name,
+                                  GValue *value,
+                                  gpointer unused G_GNUC_UNUSED)
+{
+  TpGroupMixin *mixin;
+  static GQuark q[NUM_MIXIN_DBUS_PROPERTIES] = { 0 };
+
+  if (G_UNLIKELY (q[0] == 0))
+    {
+      q[MIXIN_DP_GROUP_FLAGS] = g_quark_from_static_string ("GroupFlags");
+      q[MIXIN_DP_HANDLE_OWNERS] = g_quark_from_static_string ("HandleOwners");
+      q[MIXIN_DP_LOCAL_PENDING_MEMBERS] = g_quark_from_static_string (
+          "LocalPendingMembers");
+      q[MIXIN_DP_MEMBERS] = g_quark_from_static_string ("Members");
+      q[MIXIN_DP_REMOTE_PENDING_MEMBERS] = g_quark_from_static_string (
+          "RemotePendingMembers");
+      q[MIXIN_DP_SELF_HANDLE] = g_quark_from_static_string ("SelfHandle");
+    }
+
+  g_return_if_fail (object != NULL);
+  mixin = TP_GROUP_MIXIN (object);
+  g_return_if_fail (mixin != NULL);
+  g_return_if_fail (interface == TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP);
+  g_return_if_fail (name != 0);
+  g_return_if_fail (value != NULL);
+
+  if (name == q[MIXIN_DP_GROUP_FLAGS])
+    {
+      g_return_if_fail (G_VALUE_HOLDS_UINT (value));
+      g_value_set_uint (value, mixin->group_flags);
+    }
+  else if (name == q[MIXIN_DP_HANDLE_OWNERS])
+    {
+      g_return_if_fail (G_VALUE_HOLDS (value, TP_HASH_TYPE_HANDLE_OWNER_MAP));
+      g_value_set_boxed (value, mixin->priv->handle_owners);
+    }
+  else if (name == q[MIXIN_DP_LOCAL_PENDING_MEMBERS])
+    {
+      GPtrArray *ret = NULL;
+      gboolean success;
+
+      g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
+      success = tp_group_mixin_get_local_pending_members_with_info (object,
+          &ret, NULL);
+      g_assert (success);     /* as of 0.7.8, cannot fail */
+      g_value_take_boxed (value, ret);
+    }
+  else if (name == q[MIXIN_DP_MEMBERS])
+    {
+      GArray *ret = NULL;
+      gboolean success;
+
+      g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
+      success = tp_group_mixin_get_members (object, &ret, NULL);
+      g_assert (success);     /* as of 0.7.8, cannot fail */
+      g_value_take_boxed (value, ret);
+    }
+  else if (name == q[MIXIN_DP_REMOTE_PENDING_MEMBERS])
+    {
+      GArray *ret = NULL;
+      gboolean success;
+
+      g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
+      success = tp_group_mixin_get_remote_pending_members (object,
+          &ret, NULL);
+      g_assert (success);     /* as of 0.7.8, cannot fail */
+      g_value_take_boxed (value, ret);
+    }
+  else if (name == q[MIXIN_DP_SELF_HANDLE])
+    {
+      g_return_if_fail (G_VALUE_HOLDS_UINT (value));
+      g_value_set_uint (value, mixin->self_handle);
+    }
+  else
+    {
+      g_return_if_reached ();
+    }
+}
+
+static TpDBusPropertiesMixinPropImpl known_group_props[] = {
+    { "GroupFlags", NULL, NULL },
+    { "HandleOwners", NULL, NULL },
+    { "LocalPendingMembers", NULL, NULL },
+    { "Members", NULL, NULL },
+    { "RemotePendingMembers", NULL, NULL },
+    { "SelfHandle", NULL, NULL },
+    { NULL }
+};
+
+/**
+ * tp_group_mixin_init_dbus_properties:
+ * @cls: The class of an object with this mixin
+ *
+ * Set up #TpDBusPropertiesMixinClass to use this mixin's implementation of
+ * the Group interface's properties.
+ *
+ * This uses tp_group_mixin_get_dbus_property() as the property getter and
+ * sets up a list of the supported properties for it.
+ *
+ * Since: 0.7.10
+ */
+void
+tp_group_mixin_init_dbus_properties (GObjectClass *cls)
+{
+
+  tp_dbus_properties_mixin_implement_interface (cls,
+      TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP, tp_group_mixin_get_dbus_property,
+      NULL, known_group_props);
+}
+
+
 #define TP_EXTERNAL_GROUP_MIXIN_OBJ(o) \
     ((GObject *) g_object_get_qdata (o, \
       _external_group_mixin_get_obj_quark ()))
@@ -1583,6 +1896,72 @@ tp_external_group_mixin_finalize (GObject *obj)
   g_object_unref (obj_with_mixin);
 }
 
+/**
+ * tp_external_group_mixin_init_dbus_properties:
+ * @cls: The class of an object with this mixin
+ *
+ * Set up #TpDBusPropertiesMixinClass to use this mixin's implementation of
+ * the Group interface's properties.
+ *
+ * This uses tp_group_mixin_get_dbus_property() as the property getter and
+ * sets up a list of the supported properties for it.
+ *
+ * Since: 0.7.10
+ */
+void
+tp_external_group_mixin_init_dbus_properties (GObjectClass *cls)
+{
+
+  tp_dbus_properties_mixin_implement_interface (cls,
+      TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP,
+      tp_external_group_mixin_get_dbus_property,
+      NULL, known_group_props);
+}
+
+/**
+ * tp_external_group_mixin_get_dbus_property:
+ * @object: An object with this mixin
+ * @interface: Must be %TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP
+ * @name: A quark representing the D-Bus property name, either
+ *  "GroupFlags", "HandleOwners", "LocalPendingMembers", "Members",
+ *  "RemotePendingMembers" or "SelfHandle"
+ * @value: A GValue pre-initialized to the right type, into which to put the
+ *  value
+ * @unused: Ignored
+ *
+ * An implementation of #TpDBusPropertiesMixinGetter which assumes that the
+ * @object has the external group mixin. It can only be used for the Group
+ * interface.
+ *
+ * Since: 0.7.10
+ */
+void
+tp_external_group_mixin_get_dbus_property (GObject *object,
+                                           GQuark interface,
+                                           GQuark name,
+                                           GValue *value,
+                                           gpointer unused G_GNUC_UNUSED)
+{
+  GObject *group = TP_EXTERNAL_GROUP_MIXIN_OBJ (object);
+
+  if (group != NULL)
+    {
+      tp_group_mixin_get_dbus_property (group, interface, name, value, NULL);
+    }
+  else if (G_VALUE_HOLDS_BOXED (value))
+    {
+      /* for certain boxed types we need to supply an empty value */
+
+      if (G_VALUE_HOLDS (value, TP_HASH_TYPE_HANDLE_OWNER_MAP))
+        g_value_take_boxed (value, g_hash_table_new (NULL, NULL));
+      else if (G_VALUE_HOLDS (value, DBUS_TYPE_G_UINT_ARRAY))
+        g_value_take_boxed (value, g_array_sized_new (FALSE, FALSE,
+              sizeof (guint), 0));
+      else if (G_VALUE_HOLDS (value, TP_ARRAY_TYPE_LOCAL_PENDING_INFO_LIST))
+        g_value_take_boxed (value, g_ptr_array_sized_new (0));
+    }
+}
+
 #define EXTERNAL_OR_DIE(var) \
     GObject *var = TP_EXTERNAL_GROUP_MIXIN_OBJ ((GObject *)obj); \
     \
