@@ -839,6 +839,11 @@ tp_proxy_finalize (GObject *object)
  * #TpProxyClass), if any, being added. The intended use is for the callback
  * to call dbus_g_proxy_add_signal() on the new #DBusGProxy.
  *
+ * Since 0.7.6, to ensure correct overriding of interfaces that might be
+ * added to telepathy-glib, before calling this function you should
+ * call tp_proxy_init_known_interfaces, tp_connection_init_known_interfaces,
+ * tp_channel_init_known_interfaces etc. as appropriate for the subclass.
+ *
  * Since: 0.7.1
  */
 void
@@ -922,6 +927,8 @@ tp_proxy_class_init (TpProxyClass *klass)
   GParamSpec *param_spec;
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  tp_proxy_init_known_interfaces ();
+
   g_type_class_add_private (klass, sizeof (TpProxyPrivate));
 
   object_class->constructor = tp_proxy_constructor;
@@ -930,9 +937,6 @@ tp_proxy_class_init (TpProxyClass *klass)
   object_class->dispose = tp_proxy_dispose;
   object_class->finalize = tp_proxy_finalize;
 
-  tp_proxy_or_subclass_hook_on_interface_add (TP_TYPE_PROXY,
-      tp_cli_generic_add_signals);
-
   /**
    * TpProxy:dbus-daemon:
    *
@@ -1038,3 +1042,79 @@ tp_proxy_class_init (TpProxyClass *klass)
       _tp_marshal_VOID__UINT_INT_STRING,
       G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_INT, G_TYPE_STRING);
 }
+
+/**
+ * tp_proxy_dbus_g_proxy_claim_for_signal_adding:
+ * @proxy: a #DBusGProxy
+ *
+ * Attempt to "claim" a #DBusGProxy for addition of signal signatures.
+ * If this function has not been called on @proxy before, %TRUE is
+ * returned, and the caller may safely call dbus_g_proxy_add_signal()
+ * on @proxy. If this function has already been caled, %FALSE is
+ * returned, and the caller may not safely call dbus_g_proxy_add_signal().
+ *
+ * This is intended for use by auto-generated signal-adding functions,
+ * to allow interfaces provided as local extensions to override those in
+ * telepathy-glib without causing assertion failures.
+ *
+ * Returns: %TRUE if it is safe to call dbus_g_proxy_add_signal()
+ * Since: 0.7.6
+ */
+gboolean
+tp_proxy_dbus_g_proxy_claim_for_signal_adding (DBusGProxy *proxy)
+{
+  static GQuark q = 0;
+
+  g_return_val_if_fail (proxy != NULL, FALSE);
+
+  if (G_UNLIKELY (q == 0))
+    {
+      q = g_quark_from_static_string (
+          "tp_proxy_dbus_g_proxy_claim_for_signal_adding@0.7.6");
+    }
+
+  if (g_object_get_qdata ((GObject *) proxy, q) != NULL)
+    {
+      /* Someone else has already added signal signatures for this interface.
+       * We can't do it again or it'll cause an assertion */
+      return FALSE;
+    }
+
+  /* the proxy is just used as qdata here because it's a convenient
+   * non-NULL pointer */
+  g_object_set_qdata ((GObject *) proxy, q, proxy);
+  return TRUE;
+}
+
+static gpointer
+tp_proxy_once (gpointer data G_GNUC_UNUSED)
+{
+  GType type = TP_TYPE_PROXY;
+
+  tp_proxy_or_subclass_hook_on_interface_add (type,
+      tp_cli_generic_add_signals);
+
+  return NULL;
+}
+
+/**
+ * tp_proxy_init_known_interfaces:
+ *
+ * Ensure that the known interfaces for TpProxy have been set up.
+ * This is done automatically when necessary, but for correct
+ * overriding of library interfaces by local extensions, you should
+ * call this function before calling
+ * tp_proxy_or_subclass_hook_on_interface_add().
+ *
+ * Functions like tp_connection_init_known_interfaces and
+ * tp_channel_init_known_interfaces do this automatically.
+ *
+ * Since: 0.7.6
+ */
+void
+tp_proxy_init_known_interfaces (void)
+{
+  static GOnce once = G_ONCE_INIT;
+
+  g_once (&once, tp_proxy_once, NULL);
+}
